;;;;
;;;; file-system -- Functions and macros that help with dealing with files.
;;;;
;;;; Copyright 2006 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-general.
;;;;
;;;; octw-guile-general is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-general is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-general.  If not, see
;;;; <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: file-system.scm,v 1.1 2008-09-04 03:40:47 dean Exp $

(define-module (octw file-system))

(export extname
	line-by-line
	call-with-input-and-output-file
	call-with-input-file-or-port
	call-with-output-file-or-port)

(use-modules (ice-9 regex)
	     (ice-9 rdelim))
(use-modules (octw anaphoric)
	     (octw conditional))

(define extname
  (let ((rx (make-regexp "(\\..*)")))
    (lambda (file)
      (or
       (aand (regexp-exec rx file)
	     (match:substring it 1))
       ""))))

;;;
;;; line-by-line
;;;
;;; Reads each line from input-port and calls fn repeatedly with the
;;; following arguments:
;;;
;;;   (fn line line-number exit-continuation)
;;;
(define (line-by-line input-port fn)
  (call-with-current-continuation
   (lambda (k)
     (let loop ((i 0))
       (let ((line (read-line input-port 'concat)))
         (unless (eof-object? line)
           (fn line i k)
           (loop (+ i 1))))))))

;;;
;;; call-with-input-and-output-file
;;;
;;; Opens in-file and out-file, then calls fn with the input and
;;; output ports.
;;;
(define (call-with-input-and-output-file in-file out-file fn)
  (call-with-input-file in-file
    (lambda (in)
      (call-with-output-file out-file
	(lambda (out)
	  (fn in out))))))

;;;
;;; call-with-input-file-or-port
;;;
;;; Calls fn with an input port created by opening file-name, or if
;;; file-name is #f, call fn with the given port.
(define (call-with-input-file-or-port file-name port fn)
  (if file-name
      (call-with-input-file file-name fn)
      (fn port)))

;;;
;;; call-with-output-file-or-port
;;;
;;; Calls fn with an output port created by opening file-name, or if
;;; file-name is #f, call fn with the given port.
(define (call-with-output-file-or-port file-name port fn)
  (if file-name
      (call-with-output-file file-name fn)
      (fn port)))
