;;;;
;;;; helper -- General helper macros.
;;;;
;;;; Copyright 2007 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-general.
;;;;
;;;; octw-guile-general is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-general is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-general.  If not, see
;;;; <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: helper.scm,v 1.3 2008-09-08 18:57:19 dean Exp $

(define-module (octw helper))

(export compose-chain
        compose
        let-nth
        slot-mod!
        with-slots-helper
        with-slots
        to-exact)

(use-syntax (ice-9 syncase))

;;;
;;; compose-chain
;;;
;;; Gives you a way to unwrap deep compositions to make them more
;;; legible.  For example, this:
;;;
;;;   (enumerate* (shape-square
;;;                (row (row (make <pin> #:thickness 10) rows 0 row-pitch)
;;;                     pins-per-row pin-pitch 0)
;;;                0)
;;;               1)
;;;
;;; becomes this:
;;;
;;;   (compose-chain (p)
;;;     (make <pin> #:thickness 10)
;;;     (row p rows 0 row-pitch)
;;;     (row p pins-per-row pin-pitch 0)
;;;     (shape-square p 0)
;;;     (enumerate* p 1))
;;;
(define-syntax compose-chain
  (lambda (form)
    (syntax-case form ()
      ((_ (obj))
       (syntax
        obj))
      ((_ (obj) expr1 expr2 ...)
       (syntax
        (let ((obj expr1))
          (compose-chain (obj) expr2 ...)))))))

;;; Compose functions.  For example, this:
;;;
;;;   (lambda (d) (mils->centimils (in->mils (mm->in d))))
;;;
;;; can be written like this:
;;;
;;;   (compose mils->centimils in->mils mm->in)
;;;
(define (compose fn . rest)
  (if (null? rest)
      fn
      (lambda args
        (fn (apply (apply compose rest) args)))))

;;;
;;; let-nth
;;;
;;; Like let but defines the nth function for each binding that
;;; extracts the nth element from a list.  The nth function for the
;;; first binding extracts the first element, the nth function for the
;;; second binding extracts the second element, etc.  For example,
;;;
;;;   (let ((args '("foo.txt" "bar.txt" "baz.txt")))
;;;     (let-nth ((a (nth args))
;;;               (b (nth args))
;;;               (c (nth args)))
;;;       (format #f "a: ~A, b: ~A, c: ~A" a b c)))
;;;   => "a: foo.txt, b: bar.txt, c: baz.txt"
;;;
;;; The let-nth part in this example is equivalent to writing:
;;;
;;;     (let-nth ((a (first args))
;;;               (b (second args))
;;;               (c (third args)))
;;;       ...)
;;;
;;; Useful for dealing with command line argument lists where the
;;; argument position has some significance.
;;;
(define-syntax let-nth
  (lambda (form)
    (syntax-case form ()
      ((_ (bindings ...) body ...)
       (syntax
        (let-nth 0 (bindings ...) body ...)))
      ((_ n () body ...)
       (syntax
        (let () body ...)))
      ((_ n ((sym val) rest ...) body ...)
       (with-syntax ((next-n (datum->syntax-object
                              (syntax n)
                              (+ (syntax-object->datum (syntax n)) 1)))
                     (nth (datum->syntax-object (syntax val) 'nth)))
         (syntax
          (let-syntax ((nth (lambda (form)
                              (syntax-case form ()
                                ((_ expr)
                                 (syntax
                                  (list-ref expr n)))))))
            (let ((sym val))
              (let-nth next-n (rest ...) body ...)))))))))

;;; Like slot-set!, but you can refer to the existing value of the
;;; slot using the <> notation; e.g., this squares the existinting
;;; value of slot 'x in pont point p:
;;;
;;;   (slot-mod! p 'x (* <> <>))
(define-syntax slot-mod!
  (lambda (form)
    (syntax-case form ()
      ((_ obj slot-sym expr)
       (with-syntax ((<> (datum->syntax-object (syntax obj) '<>)))
         (syntax
          (let ((evaled-obj obj)
                (evaled-slot-sym slot-sym))
            (slot-set! evaled-obj
                       evaled-slot-sym
                       (let ((<> (slot-ref evaled-obj evaled-slot-sym)))
                         expr)))))))))

(define-syntax with-slots-helper
  (lambda (form)
    (syntax-case form ()
      ((_ obj () body ...)
       (syntax
        (begin
          body ...)))
      ((_ obj ((name slot-sym) rest ...) body ...)
       (syntax
        (let ((name (slot-ref obj slot-sym)))
          (with-slots obj (rest ...) body ...)))))))

;;; Example usage:
;;;
;;;   (with-slots pt ((x x)
;;;                   (y y))
;;;     ...)
;;;
;;; which is equivalent to:
;;;
;;;    (let ((x (slot-ref pt 'x))
;;;          (y (slot-ref pt 'y)))
;;;      ...)
;;;
(define-syntax with-slots
  (lambda (form)
    (syntax-case form ()
      ((_ obj (name-slot-pairs ...) body ...)
       (syntax
        (let ((evaled-obj obj))
          (with-slots-helper evaled-obj (name-slot-pairs ...) body ...)))))))

(define (to-exact x)
  (inexact->exact (round x)))
